package eu.tankernn.grid;

import java.awt.AWTException;
import java.awt.Image;
import java.awt.MenuItem;
import java.awt.PopupMenu;
import java.awt.SystemTray;
import java.awt.TrayIcon;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.util.Arrays;

import javax.imageio.ImageIO;
import javax.swing.JFrame;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import eu.tankernn.grid.frame.GridControlPanel;
import eu.tankernn.grid.model.ComputerModel;

public class GridControl implements Runnable {

	private static final String PROFILE_PATH = "profiles.json";
	private static final String SETTINGS_PATH = "settings.json";

	private Thread t = new Thread(this, "Polling thread");

	private int pollingSpeed = 500;

	private ComputerModel model = new ComputerModel();

	private GridControlPanel frame;
	private TrayIcon trayIcon;

	public GridControl(boolean gui) {
		readSettings();

		if (gui) {
			Image image;
			try {
				image = ImageIO.read(ClassLoader.class.getResourceAsStream("/JGC.png"));
			} catch (IOException e) {
				e.printStackTrace();
				return;
			}
			frame = new GridControlPanel(this, model);
			frame.setResizable(true);
			frame.setIconImage(image);
			frame.pack();
			frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
			frame.setVisible(true);
			addTrayIcon(image);
		}

		t.start();
	}

	private void addTrayIcon(Image image) {
		// Check the SystemTray is supported
		if (!SystemTray.isSupported()) {
			System.out.println("SystemTray is not supported");
			return;
		}
		final PopupMenu popup = new PopupMenu();

		// Create a pop-up menu components
		MenuItem openItem = new MenuItem("Open");
		openItem.addActionListener(a -> frame.setVisible(true));
		MenuItem exitItem = new MenuItem("Exit");
		exitItem.addActionListener(a -> this.exit());

		// Add components to pop-up menu
		popup.add(openItem);
		popup.addSeparator();
		popup.add(exitItem);

		trayIcon = new TrayIcon(image, "JavaGridControl", popup);
		trayIcon.setImageAutoSize(true);

		try {
			SystemTray.getSystemTray().add(trayIcon);
		} catch (AWTException e) {
			System.out.println("TrayIcon could not be added.");
		}
	}

	public void readSettings() {
		Gson gson = new GsonBuilder().create();
		try (Reader reader = new FileReader(PROFILE_PATH)) {
			Arrays.stream(gson.fromJson(reader, FanSpeedProfile[].class)).forEach(model::addProfile);
		} catch (FileNotFoundException e) {
			// Not a problem
		} catch (IOException e) {
			e.printStackTrace();
		}
		try (Reader reader = new FileReader(SETTINGS_PATH)) {
			Settings settings = gson.fromJson(reader, Settings.class);
			model.setGrid(settings.portname);
			for (int i = 0; i < 6; i++)
				model.getGrid().getFan(i).setProfile(model.getProfile(settings.fanProfiles[i]));
			pollingSpeed = settings.pollingRate;
			model.setMinSpeed(settings.minSpeed);
		} catch (FileNotFoundException e) {
			System.out.println("No config file found, using default settings.");
			for (int i = 0; i < 6; i++)
				model.getGrid().getFan(i).setProfile(model.getProfiles().get(0));
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void saveSettings() {
		Gson gson = new GsonBuilder().create();
		// Save profiles
		try (Writer writer = new FileWriter(PROFILE_PATH)) {
			gson.toJson(model.getCustomProfiles(), writer);
		} catch (IOException e) {
			e.printStackTrace();
		}
		// Save misc. settings
		try (Writer writer = new FileWriter(SETTINGS_PATH)) {
			gson.toJson(new Settings(model.getGrid().getCommunicator().getPortName(), model.getGrid().fanStream().map(f -> f.getProfile().name).toArray(String[]::new), pollingSpeed, model.getMinSpeed()), writer);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void run() {
		while (!t.isInterrupted()) {
			model.poll();
			model.compute();

			if (frame != null)
				frame.updateProperties();

			try {
				Thread.sleep(pollingSpeed);
			} catch (InterruptedException ex) {
				System.out.println("Thread was interrupted.");
				return;
			}
		}

	}

	/**
	 * 
	 * @param args the command line arguments
	 */
	public static void main(String[] args) {
		new GridControl(!Arrays.asList(args).contains("nogui"));
	}

	public void exit() {
		t.interrupt();
		model.getGrid().disconnect();
		saveSettings();
		frame.dispose();
		SystemTray.getSystemTray().remove(trayIcon);
		for (Thread t : Thread.getAllStackTraces().keySet())
			if (t.isAlive())
				System.out.println(t);
	}

	public void setPollingSpeed(int value) {
		this.pollingSpeed = value;
	}

	public int getPollingSpeed() {
		return pollingSpeed;
	}

}
